/**********************************************************************
 *
 * receptionist.cpp -- a web interface for the gsdl
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: receptionist.cpp 155 1999-02-08 01:28:04Z rjmcnab $
 *
 *********************************************************************/

/*
   $Log$
   Revision 1.5  1999/02/08 01:28:02  rjmcnab

   Got the receptionist producing something using the statusaction.

   Revision 1.4  1999/02/05 10:42:46  rjmcnab

   Continued working on receptionist

   Revision 1.3  1999/02/04 10:00:56  rjmcnab

   Developed the idea of an "action" and having them define the cgi arguments
   which they need and how those cgi arguments function.

   Revision 1.2  1999/02/04 01:17:27  rjmcnab

   Got it outputing something.


 */


#include "receptionist.h"
#include "fileutil.h"
#include "cgiutils.h"
#include <assert.h>
#include <time.h>


// this version of set_gsdlhome should be used if the receptionist
// is being run for multiple collections ("general" mode).
void receptionist::set_gsdlhome (const text_t &thegsdlhome) {
  gsdlhome = thegsdlhome;
  collectdir = thegsdlhome;
  collection = "";
}


// this version of set_gsdlhome should be used if the receptionist
// is being run for a single collection ("collection specific" mode).
void receptionist::set_gsdlhome (const text_t &thegsdlhome, const text_t &thecollection) {
  gsdlhome = thegsdlhome;
  collection = thecollection;

  // decide where collectdir is by searching for collect.cfg
  // look in $GSDLHOME/collect/collection-name/etc/collect.cfg and
  // then $GSDLHOME/etc/collect.cfg
  collectdir = filename_cat (gsdlhome, "collect");
  collectdir = filename_cat (collectdir, collection);
  text_t filename = filename_cat (collectdir, "etc");
  filename = filename_cat (filename, "collect.cfg");

  if (!file_exists(filename)) collectdir = gsdlhome;
}

// configure_actions should be called for each line in the
// configuration files to configure the actions. The configuration
// should take place after all the actions have been added.
void receptionist::configure_actions (const text_t &key, const text_tarray &cfgline) {
  actionptrmap::iterator here = actions.begin ();
  actionptrmap::iterator end = actions.end ();

  while (here != end) {
    assert ((*here).second.a != NULL);
    if ((*here).second.a != NULL) 
      (*here).second.a->configure(key, cfgline);

    here++;
  }
}

// init should be called after setgsdhome has been called.
// It returns true on success and false on failure. If false is
// returned getpage should not be called (without producing
// meaningless output), instead an error page should be
// produced by the calling code.
bool receptionist::init (ostream &logout) {
  // read in the macro files
  if (!read_macrofiles (logout)) return false;

  // defined the main cgi arguments
  if (!define_mainargs (logout)) return false;

  // add the cgi arguments from the actions
  actionptrmap::iterator here = actions.begin ();
  actionptrmap::iterator end = actions.end ();
  while (here != end) {
    assert ((*here).second.a != NULL);
    if ((*here).second.a != NULL) {
      if (!argsinfo.addarginfo (&logout, (*here).second.a->getargsinfo()))
	return false;
    }
    here++;
  }

  // create a saveconf string if there isn't one already
  if (saveconf.empty())
    saveconf = create_save_conf_str (argsinfo, logout);

  // check the saveconf string
  if (!check_save_conf_str (saveconf, argsinfo, logout))
    return false;

  // set a random seed
  srand (time(NULL));

  //  utf8outconvert.set_rzws(1);
  //  gboutconvert.set_rzws(1);

  return true;
}


// parse_cgi_args parses cgi arguments into an argument class.
// This function should be called for each page request. It returns false
// if there was a major problem with the cgi arguments.
bool receptionist::parse_cgi_args (const text_t &argstr, cgiargsclass &args, 
				   ostream &logout) {
  outconvertclass text_t2ascii;

  // get an initial list of cgi arguments
  args.clear();
  split_cgi_args (argstr, args);

  // expand the compressed argument (if there was one)
  if (!expand_save_args (argsinfo, saveconf, args, logout)) return false;

  // add the defaults
  add_default_args (argsinfo, args, logout);

  // check the main cgi arguments
  if (!check_mainargs (args, logout)) return false;

  // check the arguments for the action
  action *a = actions.getaction (args["a"]);
  if (a != NULL) {
    if (!a->check_cgiargs (args, logout)) return false;
  } else {
    // the action was not found!!
    logout << text_t2ascii << "Error: the action \"" << args["a"] 
	   << "\" could not be found.\n";
    return false;
  }

  return true;
}


// produce_cgi_page will call get_cgihead_info and 
// produce_content in the appropriate way to output a cgi header and
// the page content (if needed). If a page could not be created it
// will return false
bool receptionist::produce_cgi_page (cgiargsclass &args, ostream &contentout, 
				     ostream &logout) {
  outconvertclass text_t2ascii;

  response_t response;
  text_t response_data;

  // produce cgi header
  get_cgihead_info (args, response, response_data, logout);
  if (response == location) {
    // I've forgotten how to do this :-/
    return true;
  } else if (response == content) {
    // content response
    contentout << text_t2ascii << "Content-type: " << response_data << "\n\n";
  } else {
    // unknown response
    logout << "Error: get_cgihead_info returned an unknown response type.\n";
    return false;
  }

  // produce cgi page
  if (!produce_content (args, contentout, logout)) return false;

  // flush contentout
  contentout << flush;
  return true;
}


// get_cgihead_info determines the cgi header information for
// a set of cgi arguments. If response contains location then
// response_data contains the redirect address. If reponse
// contains content then reponse_data contains the content-type.
// Note that images can now be produced by the receptionist.
void receptionist::get_cgihead_info (cgiargsclass &args, response_t &response,
				     text_t &response_data, ostream &logout) {
  outconvertclass text_t2ascii;

  // get the action
  action *a = actions.getaction (args["a"]);
  if (a != NULL) {
    a->get_cgihead_info (args, response, response_data, logout);

  } else {
    // the action was not found!!
    logout << text_t2ascii << "Error receptionist::get_cgihead_info: the action \"" 
	   << args["a"] << "\" could not be found.\n";
    response = content;
    response_data = "text/html";
  }
}


// produce the page content
bool receptionist::produce_content (cgiargsclass &args, ostream &contentout,
				    ostream &logout) {
  outconvertclass text_t2ascii;

  // decide on the output conversion class

  // produce the page using the desired action
  action *a = actions.getaction (args["a"]);
  if (a != NULL) {
    if (!a->do_action (args, text_t2ascii, contentout, logout))
      return false;

  } else {
    // the action was not found!!
    logout << text_t2ascii << "Error receptionist::produce_content: the action \"" 
	   << args["a"] << "\" could not be found.\n";
    
    contentout << text_t2ascii << "<html>\n";
    contentout << text_t2ascii << "<head>\n";
    contentout << text_t2ascii << "<title>Error</title>\n";
    contentout << text_t2ascii << "</head>\n";
    contentout << text_t2ascii << "<body>\n";
    contentout << text_t2ascii << "<h2>Oops!</h2>\n";
    contentout << text_t2ascii << "Undefined Page. The action \"" 
	       << args["a"] << "\" could not be found.\n";
    contentout << text_t2ascii << "</body>\n";
    contentout << text_t2ascii << "</html>\n";
  }

  return true;
}


// returns the compressed argument ("e") corresponding to the argument
// list. This can be used to save preferences between sessions.
text_t receptionist::get_compressed_arg (const cgiargsclass &args) {
  return "";
}


// will read in all the macro files. If one is not found an
// error message will be written to logout and the method will
// return false.
bool receptionist::read_macrofiles (ostream &logout) {
  outconvertclass text_t2ascii;

  // redirect the error output to logout
  disp.setlogout (&logout);

  // load up the default macro files, the collection directory
  // is searched first for the file (if this is being used in
  // collection specific mode) and then the main directory
  text_t colmacrodir = filename_cat (collectdir, "macros");
  text_t gsdlmacrodir = filename_cat (gsdlhome, "macros");
  text_tarray::iterator arrhere = macrofiles.begin();
  text_tarray::iterator arrend = macrofiles.end();
  text_t filename;
  while (arrhere != arrend) {
    // filename is used as a flag to indicate whether
    // the macro file has been found
    filename.clear();

    // try in the collection directory if this is being
    // run in collection specific mode
    if (!collection.empty()) {
      filename = filename_cat (colmacrodir, *arrhere);
      if (!file_exists (filename)) filename.clear ();
    }

    // if we haven't found the macro file yet try in
    // the main macro directory
    if (filename.empty()) {
      filename = filename_cat (gsdlmacrodir, *arrhere);
      if (!file_exists (filename)) filename.clear ();
    }

    // see if we found the file or not
    if (filename.empty()) {
      logout << text_t2ascii 
	     << "Error: the macro file \"" << *arrhere << "\" could not be found.\n";
      if (collection.empty()) {
	logout << text_t2ascii
	       << "It should be in " << gsdlmacrodir << ".\n\n";
      } else {
	logout << text_t2ascii
	       << "It should be in either " << colmacrodir << " or in " 
	       << gsdlmacrodir << ".\n\n";
      }
      return false;

    } else { // found the file
      disp.loaddefaultmacros(filename);
    }

    arrhere++;
  }

  // success
  return true;
}


// Will define the main general arguments used by the receptionist.
// If an error occurs a message will be written to logout and the
// method will return false.
bool receptionist::define_mainargs (ostream &logout) {
  // create a list of cgi arguments
  cgiarginfo ainfo;

  ainfo.shortname = "e";
  ainfo.longname = "compressed arguments";
  ainfo.multiplechar = true;
  ainfo.defaultstatus = cgiarginfo::good;
  ainfo.argdefault = "";
  ainfo.savedarginfo = cgiarginfo::mustnot;
  if (!argsinfo.addarginfo (&logout, ainfo)) return false;

  ainfo.shortname = "a";
  ainfo.longname = "action";
  ainfo.multiplechar = true;
  ainfo.defaultstatus = cgiarginfo::none;
  ainfo.argdefault = "";
  ainfo.savedarginfo = cgiarginfo::must;
  if (!argsinfo.addarginfo (&logout, ainfo)) return false;

  // w=western
  ainfo.shortname = "w";
  ainfo.longname = "encoding";
  ainfo.multiplechar = true;
  ainfo.defaultstatus = cgiarginfo::weak;
  ainfo.argdefault = "w";
  ainfo.savedarginfo = cgiarginfo::must;
  if (!argsinfo.addarginfo (&logout, ainfo)) return false;
  
  ainfo.shortname = "nw";
  ainfo.longname = "new encoding";
  ainfo.multiplechar = true;
  ainfo.defaultstatus = cgiarginfo::none;
  ainfo.argdefault = "";
  ainfo.savedarginfo = cgiarginfo::mustnot;
  if (!argsinfo.addarginfo (&logout, ainfo)) return false;
  
  ainfo.shortname = "c";
  ainfo.longname = "collection";
  ainfo.multiplechar = true;
  if (collection.empty()) {
    ainfo.defaultstatus = cgiarginfo::none;
    ainfo.argdefault = "";
    ainfo.savedarginfo = cgiarginfo::must;
  } else {
    ainfo.defaultstatus = cgiarginfo::good;
    ainfo.argdefault = collection;
    ainfo.savedarginfo = cgiarginfo::can;
  }
  if (!argsinfo.addarginfo (&logout, ainfo)) return false;
  
  // 0=text+graphics, 1=text
  ainfo.shortname = "v";
  ainfo.longname = "version";
  ainfo.multiplechar = false;
  ainfo.defaultstatus = cgiarginfo::weak;
  ainfo.argdefault = "0";
  ainfo.savedarginfo = cgiarginfo::can;
  if (!argsinfo.addarginfo (&logout, ainfo)) return false;
  
  // 0=normal, 1=big
  ainfo.shortname = "f";
  ainfo.longname = "query box size";
  ainfo.multiplechar = false;
  ainfo.defaultstatus = cgiarginfo::weak;
  ainfo.argdefault = "0";
  ainfo.savedarginfo = cgiarginfo::can;
  if (!argsinfo.addarginfo (&logout, ainfo)) return false;
  
  // the interface language name should use the ISO 639
  // standard
  ainfo.shortname = "l";
  ainfo.longname = "interface language";
  ainfo.multiplechar = true;
  ainfo.defaultstatus = cgiarginfo::weak;
  ainfo.argdefault = "en";
  ainfo.savedarginfo = cgiarginfo::must;
  if (!argsinfo.addarginfo (&logout, ainfo)) return false;

  return true;
}
 
 
// check_mainargs will check all the main arguments. If a major
// error is found it will return false and no cgi page should
// be created using the arguments.
bool receptionist::check_mainargs (cgiargsclass &args, ostream &logout) {
  // if this receptionist is running in collection dependant mode
  // then it should always set the collection argument to the
  // collection
  if (!collection.empty()) args["c"] = collection;

  // argument "v" can only be 0 or 1. Use the default value
  // if it is out of range
  int arg_v = args.getintarg ("v");
  if (arg_v != 0 && arg_v != 1) {
    cgiarginfo *vinfo = argsinfo.getarginfo ("v");
    if (vinfo != NULL) args["v"] = vinfo->argdefault;
  }

  // argument "f" can only be 0 or 1. Use the default value
  // if it is out of range
  int arg_f = args.getintarg ("f");
  if (arg_f != 0 && arg_f != 1) {
    cgiarginfo *finfo = argsinfo.getarginfo ("f");
    if (finfo != NULL) args["f"] = finfo->argdefault;
  }

  return true;
}
